// Sorted Vector of Comparables

// Copyright (C) 1999 by Makoto Nagata, MicroNova <mnagata@micronova.com>.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//
// Visit http://www.micronova.com for updates

package com.micronova.util;

import netscape.util.*;

/**

   A SortedVector keeps its Comparable elements sorted in ascending order.

   * @version 1.01
   * @author Makoto Nagata, MicroNova
   * @see netscape.util.Comparable
*/

public class SortedVector extends Vector
{
    /** Adds a Comparable sorted in ascending order.  If there already exists a Comparable q such that q.equals(p), then q is replaced with p. */

    public void addComparable(Comparable p)
    {
        removeElement(p);
        addElement(p);
        sort(true);
    }

    /** Returns the segment index to which a given Comparable p belongs.
        <br>
        For an ordered sequence of N Comparables q[0] < q[1] < ... < q[N-1],<br>
        the seqment index for a Comparable p is defined as follows:<br>
        <ul>
        <li>-1 if the vector is empty</li>
        <li>0 if p < q[0]</li>
        <li>i if q[i] <= p < q[i + 1]</li>
        <li>N if q[N-1] <= p</li>
        </ul>
    */

    public int getSegmentIndex(Comparable p)
    {
        int s = size();

        if (s == 0)
        {
            return -1;
        }

        int lowIndex = 0;
        int highIndex = s - 1;

        if (p.compareTo(elementAt(lowIndex)) == -1)
        {
            return 0;
        }
        else if (p.compareTo(elementAt(highIndex)) >= 0)
        {
            return s;
        }
        else
        {
            while (lowIndex + 1L < highIndex)
            {
                int middleIndex = (lowIndex + highIndex) / 2;
     
                if (p.compareTo(elementAt(middleIndex)) >= 0)
                {
                    lowIndex = middleIndex;
                }
                else
                {
                    highIndex = middleIndex;
                }
            }
     
            return highIndex;
        }
    };
}
